//  Copyright (C) 1997, 1998 Olivetti & Oracle Research Laboratory
//
//  This file is part of the VNC system.
//
//  The VNC system is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
//  USA.
//
// If the source code for the VNC system is not available from the place 
// whence you received this file, check http://www.orl.co.uk/vnc or contact
// the authors on vnc@orl.co.uk for information on obtaining it.


// AuthDialog.cpp: implementation of the AuthDialog class.

#include "stdhdrs.h"
#include "vncviewer.h"
#include "AuthDialog.h"
#include "Exception.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

AuthDialog::AuthDialog()
{
	m_passwd[0]=__T('\0');

	// !!! (JCS) Create the NTAuth registry key
	DWORD dispos;

    if ( RegCreateKeyEx(HKEY_CURRENT_USER, NTAUTH_KEY_NAME, 0, NULL, 
 		REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, NULL, &m_hNTAuthRegKey, &dispos)  != ERROR_SUCCESS ) {
        m_hNTAuthRegKey = NULL;
    }

}

AuthDialog::~AuthDialog()
{
	 if (m_hNTAuthRegKey != NULL) {
        RegCloseKey(m_hNTAuthRegKey);
        m_hNTAuthRegKey = NULL;
    }
}

int AuthDialog::DoDialog()
{
	return DialogBoxParam(pApp->m_instance, MAKEINTRESOURCE(IDD_AUTH_DIALOG), 
		NULL, (DLGPROC) DlgProc, (LONG) this);
}

BOOL CALLBACK AuthDialog::DlgProc(  HWND hwnd,  UINT uMsg,  
									   WPARAM wParam, LPARAM lParam ) {
	// This is a static method, so we don't know which instantiation we're 
	// dealing with. But we can get a pseudo-this from the parameter to 
	// WM_INITDIALOG, which we therafter store with the window and retrieve
	// as follows:
	AuthDialog *_this = (AuthDialog *) GetWindowLong(hwnd, GWL_USERDATA);

	switch (uMsg) {

	case WM_INITDIALOG:
		{
			SetWindowLong(hwnd, GWL_USERDATA, lParam);
			_this = (AuthDialog *) lParam;
			CentreWindow(hwnd);
			
			// !!! (JCS) Set up the last username and domain
			if(_this->GetUsernameDomain(m_username, m_domain)) {
				SetDlgItemText(hwnd, IDC_USERNAME_EDIT, m_username);
				SetDlgItemText(hwnd, IDC_DOMAIN_EDIT,   m_domain);
			}
			return TRUE;
		}
	case WM_COMMAND:
		switch (LOWORD(wParam)) {
		case IDOK:
			{
				UINT res= GetDlgItemText( hwnd,  IDC_PASSWD_EDIT,
					_this->m_passwd, 256);

				// !!! (JCS)
				GetDlgItemText( hwnd,  IDC_USERNAME_EDIT,
					_this->m_username, 20);
				GetDlgItemText( hwnd,  IDC_DOMAIN_EDIT,
					_this->m_domain, 20);

				// !!! (JCS)
				AddUsernameDomain(m_username, m_domain);
				
				EndDialog(hwnd, TRUE);

				return TRUE;
			}
		case IDCANCEL:
			EndDialog(hwnd, FALSE);
			return TRUE;
		}
		break;
	case WM_DESTROY:
		EndDialog(hwnd, FALSE);
		return TRUE;
	}
	return 0;
}

// !!! (JCS)
// Add the last used username and NT domain to the registry for future use
void AuthDialog::AddUsernameDomain(LPTSTR username, LPTSTR domain) 
{
	TCHAR valname[20];

	if (m_hNTAuthRegKey == NULL) return;

	_tcscpy(valname, "LastUsername");
	RegSetValueEx(m_hNTAuthRegKey, valname, NULL, REG_SZ,
		(CONST BYTE*) username, (_tcslen(username)+1)*sizeof(TCHAR));

	_tcscpy(valname, "LastNTDomain");
	RegSetValueEx(m_hNTAuthRegKey, valname, NULL, REG_SZ,
		(CONST BYTE*) domain, (_tcslen(domain)+1)*sizeof(TCHAR));

}

// Read the last used username and NT domain from the registry
int AuthDialog::GetUsernameDomain(LPCTSTR  username, LPCTSTR  domain)
{
	TCHAR valname[20];
	DWORD valtype, dwbuflen;

	if (m_hNTAuthRegKey == NULL) return 0;
	
	dwbuflen = 21;
	_tcscpy(valname, "LastUsername");
	if ( RegQueryValueEx (m_hNTAuthRegKey, valname, NULL, &valtype,
		             (LPBYTE) username, &dwbuflen) != ERROR_SUCCESS)
		return 0;

	_tcscpy(valname, "LastNTDomain");
	if ( RegQueryValueEx (m_hNTAuthRegKey, valname, NULL, &valtype,
		             (LPBYTE) domain, &dwbuflen) != ERROR_SUCCESS)
		return 0;
	
	return 1;
}